/*!
    \file object_manager.h
  *  this manager will ceep track of the objects  in the world

    This is specially designed to:
    - Give an interface to the world data
    - separate the world data from the world build,update,draw process
    - recycle deleted objects: specific for Projectils since there is a lot of world entity creation/deletion (and this needs a lot of time)
    - control the garbage collector

    TO ADD SUPPORT FOR A CLASS do the following steps:
    1. include the hader file : #include "class_id.h"
    2. add the class to the type enum classID {}; in class_id.h
    3. define a function void mCache( ClassName ) in class ObjectManager

*/


#ifndef _OBJECT_MANAGER_H
#define _OBJECT_MANAGER_H

#include "base_object.h"
#include "class_id.h"

template<class T> class tList;
class GarbageCollector;


/**
 * Creates a FastFactory to a Loadable FastFactory.
 */
#define CREATE_FAST_FACTORY(CLASS_NAME, CLASS_ID) \
    tFastFactory<CLASS_NAME>* global_##CLASS_NAME##_FastFactory = new tFastFactory<CLASS_NAME>(#CLASS_NAME, CLASS_ID)

//! A struct, that holds Lists of Objects of a certain type.
typedef struct FastObjectMember
{
  BaseObject*          objectPointer;

  FastObjectMember*    next;
};

//! The FastFactory is a fast loadable object creator, and Dynamic List of dead object handler.
class FastFactory : public BaseObject {

  public:
    FastFactory (const char* fastFactoryName, ClassID classID);
    virtual ~FastFactory ();


    BaseObject* resurect(ClassID classID);
    void kill(ClassID classID, BaseObject* object);

    virtual BaseObject* fabricate(ClassID classID) = NULL;

    static void registerFastFactory(FastFactory* fastFactory);
    static void registerFastFactory(const char* fastFactoryName, ClassID classID);

    /** @returns the first FastFactory */
    static FastFactory* getFirst() { return FastFactory::first; };

  protected:
    /** sets the Next factory in the list @param nextFactory the next factory */
    inline void setNext( FastFactory* nextFastFactory) { this->next = nextFastFactory; };
    /** @returns the next FastFactory */
    FastFactory* getNext() const { return this->next; };

  protected:
    ClassID               storedClassID;        //!< The classID of the specified class.

  private:
    static FastFactory*   first;                //!< A pointer to the first FastFactory.

    FastFactory*          next;                 //!< pointer to the next FastFactory.
};

/**
 *  a FastFactory that is able to load any kind of Object from a ClassID
 * (this is a Functor)
 */
template<class T> class tFastFactory : public FastFactory
{
  public:
    tFastFactory(const char* fastFactoryName, ClassID fastFactory);

  private:
    virtual BaseObject* fabricate(ClassID classID);
};

/**
 * construnts a FastFactory with
 * @param fastFactoryName the name of the FastFactory
 * @param fastFactory the ID of the class
 */
template<class T>
    tFastFactory<T>::tFastFactory(const char* fastFactoryName, ClassID fastFactory) : FastFactory(fastFactoryName, fastFactory)
{
  PRINTF(5)("Class: %s loadable as a FastFactory\n", this->getName());
}

template<class T>
BaseObject* tFastFactory<T>::fabricate(ClassID classID)
{
  if (this->storedClassID == classID)
    return new T ();
  else if( getNext() != NULL)
    return getNext()->fabricate(classID);
  else
    return NULL;
}




////////////////////
// OBJECT MANAGER //
////////////////////

//! the object manager itself
class ObjectManager : public BaseObject {

 public:
  virtual ~ObjectManager();
  /** @returns a Pointer to the only object of this Class */
  inline static ObjectManager* getInstance() { if (!singletonRef) singletonRef = new ObjectManager();  return singletonRef; };

  void registerClass(ClassID classID);

  BaseObject* resurect();
  void kill(BaseObject* object);


  void debug() const;

 private:
  ObjectManager();

 private:
  static ObjectManager*      singletonRef;          //!< The singleton reference to the only reference of this class

};



#endif /* _OBJECT_MANAGER_H */
